package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.framework.obs.FileUtil;
import com.ruoyi.zdsz.domain.vo.GExportsVo;
import com.ruoyi.zdsz.service.IZBuildingService;
import lombok.RequiredArgsConstructor;
import lombok.Value;
import org.mybatis.logging.Logger;
import org.mybatis.logging.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.zdsz.domain.bo.ZAreaBo;
import com.ruoyi.zdsz.domain.vo.ZAreaVo;
import com.ruoyi.zdsz.domain.ZArea;
import com.ruoyi.zdsz.mapper.ZAreaMapper;
import com.ruoyi.zdsz.service.IZAreaService;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 小区管理Service业务层处理
 *
 * @author ruoyi
 * @date 2023-12-27
 */
@RequiredArgsConstructor
@Service
public class ZAreaServiceImpl implements IZAreaService {

    private static final Logger logger = LoggerFactory.getLogger(ZAreaServiceImpl.class);

    @Value("${filePath}")
    private String path;

    @Resource
    private ZAreaMapper baseMapper;


    /**
     * 小区下拉
     */
    @Override
    public List<ZAreaVo> getArea(String district) {
        return baseMapper.getArea(district);
    }

    /**
     * 查询小区管理
     */
    @Override
    public ZAreaVo queryById(String id) {
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询小区管理列表
     */
    @Override
    public TableDataInfo<ZAreaVo> queryPageList(ZAreaBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZArea> lqw = buildQueryWrapper(bo);
        Page<ZAreaVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询小区管理列表
     */
    @Override
    public List<ZAreaVo> queryList(ZAreaBo bo) {
        LambdaQueryWrapper<ZArea> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZArea> buildQueryWrapper(ZAreaBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZArea> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), ZArea::getName, bo.getName());
        lqw.eq(StringUtils.isNotBlank(bo.getScale()), ZArea::getScale, bo.getScale());
        lqw.eq(bo.getReleaseIs() != null, ZArea::getReleaseIs, bo.getReleaseIs());
        lqw.eq(StringUtils.isNotBlank(bo.getReleaseBy()), ZArea::getReleaseBy, bo.getReleaseBy());
        lqw.eq(StringUtils.isNotBlank(bo.getDistrict()), ZArea::getDistrict, bo.getDistrict());
        lqw.eq(StringUtils.isNotBlank(bo.getCalculateDistrict()), ZArea::getCalculateDistrict, bo.getCalculateDistrict());
        lqw.eq(StringUtils.isNotBlank(bo.getBelongingPack()), ZArea::getBelongingPack, bo.getBelongingPack());
        return lqw;
    }

    /**
     * 新增小区管理
     */
    @Override
    public Boolean insertByBo(ZAreaBo bo) {
        ZArea add = BeanUtil.toBean(bo, ZArea.class);
        validEntityBeforeSave(add);
        add.setReleaseIs("001");
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改小区管理
     */
    @Override
    public Boolean updateByBo(ZAreaBo bo) {
        ZArea update = BeanUtil.toBean(bo, ZArea.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZArea entity) {
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除小区管理
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if (isValid) {

        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }

    /**
     * 导出所属包功能(名称查询)
     */
    @Override
    public String getDictNameOfAffiliatedPackage(ZAreaBo bo) {
        return baseMapper.getDictNameOfAffiliatedPackage(bo);
    }

    /**
     * 导出所属包功能(数据查询)
     */
    @Override
    public List<GExportsVo> getExport(ZAreaBo bo) {
        return baseMapper.getExport(bo);
    }

    /**
     * 图片下载
     */
    @Override
    public void upload(HttpServletRequest request, HttpServletResponse response, String name) {
        // 设置文件夹时间戳和路径位置
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String format = sdf.format(new Date());
        // 文件夹的路径，写入服务器的 /tmp 文件夹下
        String filePath = path + name;
        // 导出的zip路径
        String zipFilePath = path + "tmp_" + format + ".zip";
        // 导出的zip名字
        String zipName = "PHOTO_" + format + ".zip";

        // 调用工具类压缩文件夹
        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(new File(zipFilePath));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        FileUtil.toZip(filePath, fileOutputStream, true);

        // 调用工具类设置响应格式
        try {
            FileUtil.downLoadFile(request, response, zipName, zipFilePath);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("导出失败~"+e);
        }
    }
}
